@@ -68,8 +68,13 @@ constexpr bool isValidHopperMOESpecialisation() {
   return (cutlass::platform::is_same<T, WeightType>::value ||
           (cutlass::platform::is_same<cutlass::uint4b_t, WeightType>::value &&
            cutlass::platform::is_same<T, __nv_fp8_e4m3>::value) ||
+#ifdef ENABLE_FP4
           (cutlass::platform::is_same<__nv_fp4_e2m1, WeightType>::value &&
-           !cutlass::platform::is_same<T, __nv_fp8_e4m3>::value))
+           !cutlass::platform::is_same<T, __nv_fp8_e4m3>::value)
+#else
+          false
+#endif
+              )
 
 #ifdef ENABLE_FP4
          && !cutlass::platform::is_same<T, __nv_fp4_e2m1>::value
