@@ -35,17 +35,28 @@ namespace torch_ext {
 // mxfp4: sfVecSize = 32, sfUseUE8M0 = true
 // alignment: sfVecSize
 // isSfSwizzledLayout: bool, if true, the scale factors are stored in swizzled layout, otherwise in
-// linear layout. See FP4QuantizationSFLayout enum for more details about the two layouts. returns
+// linear layout. See QuantizationSFLayout enum for more details about the two layouts. returns
 // self_fp4, self_block_scale_factors self_fp4: [M, K / 2], FLOAT4_E2M1X2 self_block_scale_factors:
 // ceil(M / 128) * 128 * ceil(K / sfVecSize / 4) * 4, SF_DTYPE (UE4M3 or UE8M0)
 std::tuple<at::Tensor, at::Tensor> fp4_quantize(at::Tensor const& self,
-                                                at::Tensor const& globalScale, int64_t sfVecSize,
-                                                bool sfUseUE8M0, bool isSfSwizzledLayout,
-                                                bool isSf8x4Layout, bool enable_pdl) {
+                                                std::optional<at::Tensor> const& globalScale,
+                                                int64_t sfVecSize, bool sfUseUE8M0,
+                                                bool isSfSwizzledLayout, bool isSf8x4Layout,
+                                                bool enable_pdl) {
   CHECK_TH_CUDA(self);
   CHECK_CONTIGUOUS(self);
-  CHECK_INPUT_TYPE(globalScale, c10::ScalarType::Float);
-  TORCH_CHECK(sfVecSize == 16 || sfVecSize == 32, "sfVecSize can only be 16 or 32");
+  if (sfUseUE8M0) {
+    TORCH_CHECK(sfVecSize == 32, "sfVecSize can only be 32, when sfUseUE8M0 is true");
+  } else {
+    TORCH_CHECK(globalScale.has_value(), "globalScale is required when sfUseUE8M0 is false");
+    // CHECK_INPUT_AND_TYPE(globalScale.value(), torch::kFloat32);
+    TORCH_CHECK(sfVecSize == 16, "sfVecSize can only be 16, when sfUseUE8M0 is false");
+  }
+
+  float* globalScalePtr{nullptr};
+  if (globalScale.has_value()) {
+    globalScalePtr = globalScale->data_ptr<float>();
+  }
 
   auto const& inputShape = self.sizes();
   auto const& rank = inputShape.size();
@@ -64,28 +75,23 @@ std::tuple<at::Tensor, at::Tensor> fp4_quantize(at::Tensor const& self,
   at::Tensor valueE2M1 =
       at::detail::empty_cuda(outputShape, FLOAT4_E2M1X2, self.device(), /* stride */ std::nullopt);
 
-  int64_t SFSize =
-      isSfSwizzledLayout
-          ? tensorrt_llm::computeFP4SwizzledLayoutSFSize(m, k / sfVecSize, isSf8x4Layout ? 8 : 128)
-          : tensorrt_llm::computeFP4LinearLayoutSFSize(m, k / sfVecSize);
+  int64_t SFSize = isSfSwizzledLayout ? tensorrt_llm::computeSwizzledLayoutSFSize(
+                                            m, k / sfVecSize, isSf8x4Layout ? 8 : 128)
+                                      : tensorrt_llm::computeLinearLayoutSFSize(m, k / sfVecSize);
 
   at::Tensor scaleFP8SF = at::detail::empty_cuda({SFSize}, SF_DTYPE, self.device(),
                                                  /* stride */ std::nullopt);  // 1D tensor
 
   const thread_local int mMultiProcessorCount = tensorrt_llm::common::getMultiProcessorCount();
 
-  auto layout = tensorrt_llm::FP4QuantizationSFLayout::LINEAR;
-  if (isSf8x4Layout) {
-    TORCH_CHECK(isSfSwizzledLayout, "8x4layout must be swizzled layout");
-    layout = tensorrt_llm::FP4QuantizationSFLayout::SWIZZLED_8x4;
-  } else {
-    layout = isSfSwizzledLayout ? tensorrt_llm::FP4QuantizationSFLayout::SWIZZLED_128x4
-                                : tensorrt_llm::FP4QuantizationSFLayout::LINEAR;
-  }
+  auto layout = tensorrt_llm::QuantizationSFLayout::LINEAR;
+  layout = isSfSwizzledLayout ? (isSf8x4Layout ? tensorrt_llm::QuantizationSFLayout::SWIZZLED_8x4
+                                               : tensorrt_llm::QuantizationSFLayout::SWIZZLED_128x4)
+                              : tensorrt_llm::QuantizationSFLayout::LINEAR;
 
 #define LAUNCH_FP4_QUANTIZE_KERNEL(T, SF_VEC_SIZE)                                                 \
   tensorrt_llm::kernels::invokeFP4Quantization<T, SF_VEC_SIZE>(                                    \
-      m, k, reinterpret_cast<T*>(self.data_ptr()), globalScale.data_ptr<float>(),                  \
+      1, m, k, reinterpret_cast<T*>(self.data_ptr()), globalScalePtr,                              \
       reinterpret_cast<int64_t*>(valueE2M1.data_ptr()),                                            \
       reinterpret_cast<int32_t*>(scaleFP8SF.data_ptr()), sfUseUE8M0, layout, mMultiProcessorCount, \
       enable_pdl, at::cuda::getCurrentCUDAStream(self.get_device()));
