@@ -8,7 +8,7 @@ _data_dir = _root / "flashinfer" / "data"
 _aot_ops_dir = _root / "aot-ops"
 _aot_ops_package_dir = _root / "build" / "aot-ops-package-dir"
 
-_requires_for_aot = ["torch", "ninja", "numpy"]
+_requires_for_aot = ["torch", "ninja", "numpy", "requests"]
 
 
 def _rm_aot_ops_package_dir():
