@@ -21,7 +21,11 @@ import torch
 
 from .jit import JitSpec
 from .jit import env as jit_env
-from .jit import gen_batch_mla_module, gen_jit_spec, sm100a_nvcc_flags
+from .jit import (
+    gen_batch_mla_module,
+    gen_jit_spec,
+    current_compilation_context,
+)
 from .utils import MaskMode, check_shape_dtype_device, determine_mla_backend
 
 
@@ -55,13 +59,16 @@ def _check_cutlass_shape(q_nope_pe, ckv_kpe_cache, kv_len, page_table):
 
 
 def gen_mla_module() -> JitSpec:
+    nvcc_flags = current_compilation_context.get_nvcc_flags_list(
+        supported_major_versions=[10, 11]
+    )
     return gen_jit_spec(
         "mla",
         [
             jit_env.FLASHINFER_CSRC_DIR / "cutlass_mla.cu",
             jit_env.FLASHINFER_CSRC_DIR / "flashinfer_mla_ops.cu",
         ],
-        extra_cuda_cflags=sm100a_nvcc_flags,
+        extra_cuda_cflags=nvcc_flags,
     )
 
 
