@@ -19,6 +19,7 @@
 #include <ATen/cuda/CUDAContext.h>
 #include <c10/cuda/CUDAGuard.h>
 #include <c10/cuda/CUDAStream.h>
+#include <cuda_fp8.h>
 #include <limits.h>
 #include <stdint.h>
 #ifndef _WIN32  // Linux
@@ -195,19 +196,6 @@ inline bool getEnvUseTileSizeKv64ForTrtllmGen() {
   static bool const useTileSizeKv64 = getBoolEnv("TRTLLM_GEN_ENABLE_TILE_SIZE_KV64");
   return useTileSizeKv64;
 }
-
-inline bool getEnvEnablePDL() {
-  static std::once_flag flag;
-  static bool enablePDL = false;
-
-  std::call_once(flag, [&]() {
-    if (getSMVersion() >= 90) {
-      // PDL will be enabled by setting the env variables `TRTLLM_ENABLE_PDL` to `1`
-      enablePDL = getBoolEnv("TRTLLM_ENABLE_PDL");
-    }
-  });
-  return enablePDL;
-}
 template <typename T>
 inline __device__ __host__ T divUp(T m, T n) {
   return (m + n - 1) / n;
@@ -271,6 +259,11 @@ struct TypeToDataType<uint8_t> {
   static constexpr Data_type value = Data_type::DATA_TYPE_E4M3;
 };
 
+template <>
+struct TypeToDataType<__nv_fp8_e4m3> {
+  static constexpr Data_type value = Data_type::DATA_TYPE_E4M3;
+};
+
 static inline size_t get_size_in_bytes(size_t n, Data_type dtype) {
   switch (dtype) {
     case DATA_TYPE_FP32:
@@ -326,4 +319,6 @@ constexpr int32_t kSM_86 = 86;
 constexpr int32_t kSM_89 = 89;
 constexpr int32_t kSM_90 = 90;
 constexpr int32_t kSM_100 = 100;
+constexpr int32_t kSM_100f = 10100;
+constexpr int32_t kSM_103 = 103;
 constexpr int32_t kSM_120 = 120;
