@@ -136,9 +136,6 @@ struct KernelParams {
   int32_t mNumHeadsQPerKv;
   // The hidden size of O.
   int64_t mNumHiddenEltsO;
-  // The number of MTP tokens per sequence. Assume that all requests have the same numMtpTokens
-  // without paddings.
-  int32_t mNumMtpTokens;
   // The total number of pages in the paged-kv memory pool.
   int32_t mNumPagesInMemPool;
   // The output scale for FP8 quantization.
@@ -687,7 +684,6 @@ struct KernelParams {
     params.mMaxNumCtasKv = maxNumCtasKv;
     params.mMaxNumPagesPerSeqKv = options.mMaxNumPagesPerSeqKv;
     // TODO: just use mMaxSeqLenQ for number of MTP tokens.
-    params.mNumMtpTokens = options.mMaxSeqLenQ;
     params.mSumOfSeqLensQ = options.mSumOfSeqLensQ;
     params.mSumOfSeqLensKv = options.mSumOfSeqLensKv;
     params.mBatchSize = options.mBatchSize;
