@@ -16,6 +16,7 @@
  */
 #pragma once
 
+#include <optional>
 #include <set>
 #include <sstream>
 
@@ -31,20 +32,26 @@
 #else
 #include <iostream>
 
+template <typename T>
+void printArgs(T arg) {
+#ifdef TLLM_GEN_DEBUG
+  std::cout << arg;
+#endif
+}
+
 template <typename T, typename... Args>
 void printArgs(T first, Args... args) {
-#ifdef TLLM_GEN_DEBUG
-  std::cout << first;
+  printArgs(first);
   if constexpr (sizeof...(args) > 0) {
-    std::cout << " ";
+    printArgs(", ");
     printArgs(args...);
   }
-#endif
 }
 
 #define TLLM_CHECK_ERROR(cond, ...) \
   if (!(cond)) {                    \
     printArgs(__VA_ARGS__);         \
+    printArgs("\n");                \
     return false;                   \
   }
 
@@ -55,6 +62,7 @@ void printArgs(T first, Args... args) {
 #define TLLM_CHECK_WARNING(cond, ...) \
   if (!(cond)) {                      \
     printArgs(__VA_ARGS__);           \
+    printArgs("\n");                  \
     return false;                     \
   }
 
@@ -62,7 +70,7 @@ void printArgs(T first, Args... args) {
 
 #define TLLM_LOG_INFO(...) TLLM_CHECK_WARNING(false, __VA_ARGS__)
 
-#endif
+#endif  // TLLM_GEN_EXPORT_INTERFACE
 
 namespace gemm {
 
@@ -95,13 +103,14 @@ struct GemmOptions {
               int mmaM, int mmaN, bool mockAllReduce, int n, int numSlicesForSplitK,
               int numSlicesForSliceK, int numStages, int numStagesMma,
               int numStagesMmaWithinWorkTile, int numStagesMmaAcrossWorkTile, int numStagesWorkId,
-              bool outputDebugTensors, bool patchF2fp, bool useShuffledMatrixA, bool sliceK,
-              SplitK splitK, bool transposeMmaOutput, int tileM, int tileN, int tileK,
-              bool useUnrollLoop2xForMma, bool useCustomMmaSchedule,
-              bool useHoistTryWaitForCustomMmaSchedule, bool useDeepSeekFp8, bool usePerTokenSfA,
-              bool usePerTokenSfB, bool useTmaStore, bool useTwoTmaLoadWarps, bool useTwoMmaWarps,
+              bool outputDebugTensors, bool patchF2fp, std::optional<int32_t> sfBlockSizeA,
               tg::SfLayout sfLayoutA, tg::SfLayout sfLayoutB, tg::SfLayout sfLayoutC,
-              int sfReshapeFactor, TileScheduler tileScheduler)
+              int sfReshapeFactor, bool sliceK, SplitK splitK, int tileK, int tileM, int tileN,
+              TileScheduler tileScheduler, bool transposeMmaOutput, bool useCustomMmaSchedule,
+              bool useDeepSeekFp8, bool useHoistTryWaitForCustomMmaSchedule, bool usePerTokenSfA,
+              bool usePerTokenSfB, bool useShuffledMatrixA, bool useTmaStore,
+              bool useTwoTmaLoadWarps, bool useTwoMmaWarps, bool useUnrollLoop2xForMma,
+              int worldSize)
       : mAllReduceAlgo{allReduceAlgo},
         mBiasType{biasType},
         mBlockK(blockK),
@@ -148,27 +157,29 @@ struct GemmOptions {
         mNumStagesWorkId{numStagesWorkId},
         mOutputDebugTensors{outputDebugTensors},
         mPatchF2fp{patchF2fp},
-        mUseShuffledMatrixA{useShuffledMatrixA},
-        mSliceK{sliceK},
-        mSplitK{splitK},
-        mTransposeMmaOutput{transposeMmaOutput},
-        mTileM{tileM},
-        mTileN{tileN},
-        mTileK{tileK},
-        mUseUnrollLoop2xForMma{useUnrollLoop2xForMma},
-        mUseCustomMmaSchedule{useCustomMmaSchedule},
-        mUseHoistTryWaitForCustomMmaSchedule{useHoistTryWaitForCustomMmaSchedule},
-        mUseDeepSeekFp8{useDeepSeekFp8},
-        mUsePerTokenSfA{usePerTokenSfA},
-        mUsePerTokenSfB{usePerTokenSfB},
-        mUseTmaStore{useTmaStore},
-        mUseTwoTmaLoadWarps{useTwoTmaLoadWarps},
-        mUseTwoMmaWarps{useTwoMmaWarps},
+        mSfBlockSizeA{sfBlockSizeA},
         mSfLayoutA{sfLayoutA},
         mSfLayoutB{sfLayoutB},
         mSfLayoutC{sfLayoutC},
         mSfReshapeFactor{sfReshapeFactor},
-        mTileScheduler{tileScheduler} {}
+        mSliceK{sliceK},
+        mSplitK{splitK},
+        mTileK{tileK},
+        mTileM{tileM},
+        mTileN{tileN},
+        mTileScheduler{tileScheduler},
+        mTransposeMmaOutput{transposeMmaOutput},
+        mUseCustomMmaSchedule{useCustomMmaSchedule},
+        mUseDeepSeekFp8{useDeepSeekFp8},
+        mUseHoistTryWaitForCustomMmaSchedule{useHoistTryWaitForCustomMmaSchedule},
+        mUsePerTokenSfA{usePerTokenSfA},
+        mUsePerTokenSfB{usePerTokenSfB},
+        mUseShuffledMatrixA{useShuffledMatrixA},
+        mUseTmaStore{useTmaStore},
+        mUseTwoTmaLoadWarps{useTwoTmaLoadWarps},
+        mUseTwoMmaWarps{useTwoMmaWarps},
+        mUseUnrollLoop2xForMma{useUnrollLoop2xForMma},
+        mWorldSize{worldSize} {}
 
   // The all-reduce algorithm.
   AllReduceAlgo mAllReduceAlgo{AllReduceAlgo::None};
@@ -274,40 +285,8 @@ struct GemmOptions {
   bool mOutputDebugTensors{false};
   // Patch float conversions.
   bool mPatchF2fp{false};
-  // Reorder rows/cols in the A matrix for the better memory accesses in the M-major epilogue.
-  bool mUseShuffledMatrixA{false};
-  // Slice-K implementation to use TileM dimension for TileK.
-  bool mSliceK{false};
-  // The location of the exchange for split-K (it's None when split-K is disabled).
-  SplitK mSplitK{SplitK::None};
-  // Save output of MMA in M-major format.
-  bool mTransposeMmaOutput{false};
-  // M tile dimension of GEMM.
-  int mTileM{128};
-  // N tile dimension of GEMM.
-  int mTileN{32};
-  // K tile dimension of GEMM.
-  int mTileK{16};
-  // Whether to unroll the loop by 2x.
-  bool mUseUnrollLoop2xForMma{true};
-  // Use custom MMA schedule optimized for low-latency.
-  bool mUseCustomMmaSchedule{false};
-  // The purpose of hoisting trywaits is to opportunistically peek at the availability of the next
-  // k-block. It benefits when the next k-block is already available and thus sustaining the
-  // momentum, but it adds latency to the first k-block for smaller k-loop.
-  bool mUseHoistTryWaitForCustomMmaSchedule{false};
-  // Use DeepSeek Fp8.
-  bool mUseDeepSeekFp8{false};
-  // Apply per-token scales from A
-  bool mUsePerTokenSfA{false};
-  // Apply per-token scales from B
-  bool mUsePerTokenSfB{false};
-  // Use TMA to store the result.
-  bool mUseTmaStore{true};
-  // Use two different warps for A and B matrix load.
-  bool mUseTwoTmaLoadWarps{false};
-  // Use two different warps for MMA tasks. Applicable only to DeepSeek FP8.
-  bool mUseTwoMmaWarps{false};
+  // Block size of A. For dtypeA == E2m1 and dtypeB == E4m3.
+  std::optional<int32_t> mSfBlockSizeA{std::nullopt};
   // Scale factors layout for A.
   tg::SfLayout mSfLayoutA{tg::SfLayout::R128c4};
   // Scale factors layout for B.
@@ -320,13 +299,56 @@ struct GemmOptions {
   // But it reduces the number of L2 requests under the hood and potentially improves perf.
   // Applies to layout 8x4 only.
   int mSfReshapeFactor{1};
+  // Slice-K implementation to use TileM dimension for TileK.
+  bool mSliceK{false};
+  // The location of the exchange for split-K (it's None when split-K is disabled).
+  SplitK mSplitK{SplitK::None};
+  // K tile dimension of GEMM.
+  int mTileK{16};
+  // M tile dimension of GEMM.
+  int mTileM{128};
+  // N tile dimension of GEMM.
+  int mTileN{32};
   // Tile scheduler type.
   TileScheduler mTileScheduler{TileScheduler::Static};
+  // Save output of MMA in M-major format.
+  bool mTransposeMmaOutput{false};
+  // Use custom MMA schedule optimized for low-latency.
+  bool mUseCustomMmaSchedule{false};
+  // Use DeepSeek Fp8.
+  bool mUseDeepSeekFp8{false};
+  // The purpose of hoisting trywaits is to opportunistically peek at the availability of the next
+  // k-block. It benefits when the next k-block is already available and thus sustaining the
+  // momentum, but it adds latency to the first k-block for smaller k-loop.
+  bool mUseHoistTryWaitForCustomMmaSchedule{false};
+  // Apply per-token scales from A
+  bool mUsePerTokenSfA{false};
+  // Apply per-token scales from B
+  bool mUsePerTokenSfB{false};
+  // Reorder rows/cols in the A matrix for the better memory accesses in the M-major epilogue.
+  bool mUseShuffledMatrixA{false};
+  // Use TMA to store the result.
+  bool mUseTmaStore{true};
+  // Use two different warps for A and B matrix load.
+  bool mUseTwoTmaLoadWarps{false};
+  // Use two different warps for MMA tasks. Applicable only to DeepSeek FP8.
+  bool mUseTwoMmaWarps{false};
+  // Whether to unroll the loop by 2x.
+  bool mUseUnrollLoop2xForMma{true};
+  // World size for all-reduce.
+  int mWorldSize{1};
 };
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
-enum class SmVersion { Sm90a, Sm100a };
+enum class SmVersion { Sm90a, Sm100a, Sm100f, Sm103a };
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+
+inline bool isSmVersionBlackwell(SmVersion smVersion) {
+  return smVersion == SmVersion::Sm100a || smVersion == SmVersion::Sm100f ||
+         smVersion == SmVersion::Sm103a;
+}
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 //
@@ -436,24 +458,11 @@ inline std::string dumpOptions(GemmOptions const& options) {
   ss << "mNumStagesWorkId=" << options.mNumStagesWorkId << "," << std::endl;
   ss << "mOutputDebugTensors=" << options.mOutputDebugTensors << "," << std::endl;
   ss << "mPatchF2fp=" << options.mPatchF2fp << "," << std::endl;
-  ss << "mUseShuffledMatrixA=" << options.mUseShuffledMatrixA << "," << std::endl;
-  ss << "mSliceK=" << options.mSliceK << "," << std::endl;
-  ss << "mSplitK=" << "gemm::SplitK(" << static_cast<int32_t>(options.mSplitK) << ")" << ","
-     << std::endl;
-  ss << "mTransposeMmaOutput=" << options.mTransposeMmaOutput << "," << std::endl;
-  ss << "mTileM=" << options.mTileM << "," << std::endl;
-  ss << "mTileN=" << options.mTileN << "," << std::endl;
-  ss << "mTileK=" << options.mTileK << "," << std::endl;
-  ss << "mUseUnrollLoop2xForMma=" << options.mUseUnrollLoop2xForMma << "," << std::endl;
-  ss << "mUseCustomMmaSchedule=" << options.mUseCustomMmaSchedule << "," << std::endl;
-  ss << "mUseHoistTryWaitForCustomMmaSchedule=" << options.mUseHoistTryWaitForCustomMmaSchedule
-     << "," << std::endl;
-  ss << "mUseDeepSeekFp8=" << options.mUseDeepSeekFp8 << "," << std::endl;
-  ss << "mUsePerTokenSfA=" << options.mUsePerTokenSfA << "," << std::endl;
-  ss << "mUsePerTokenSfB=" << options.mUsePerTokenSfB << "," << std::endl;
-  ss << "mUseTmaStore=" << options.mUseTmaStore << "," << std::endl;
-  ss << "mUseTwoTmaLoadWarps=" << options.mUseTwoTmaLoadWarps << "," << std::endl;
-  ss << "mUseTwoMmaWarps=" << options.mUseTwoMmaWarps << "," << std::endl;
+  if (options.mSfBlockSizeA.has_value()) {
+    ss << "mSfBlockSizeA=" << options.mSfBlockSizeA.value() << "," << std::endl;
+  } else {
+    ss << "mSfBlockSizeA=" << "std::nullopt" << ", " << std::endl;
+  }
   ss << "mSfLayoutA=" << "trtllm::gen::SfLayout(" << static_cast<int32_t>(options.mSfLayoutA) << ")"
      << "," << std::endl;
   ss << "mSfLayoutB=" << "trtllm::gen::SfLayout(" << static_cast<int32_t>(options.mSfLayoutB) << ")"
@@ -461,8 +470,27 @@ inline std::string dumpOptions(GemmOptions const& options) {
   ss << "mSfLayoutC=" << "trtllm::gen::SfLayout(" << static_cast<int32_t>(options.mSfLayoutC) << ")"
      << "," << std::endl;
   ss << "mSfReshapeFactor=" << options.mSfReshapeFactor << "," << std::endl;
+  ss << "mSliceK=" << options.mSliceK << "," << std::endl;
+  ss << "mSplitK=" << "gemm::SplitK(" << static_cast<int32_t>(options.mSplitK) << ")" << ","
+     << std::endl;
+  ss << "mTileK=" << options.mTileK << "," << std::endl;
+  ss << "mTileM=" << options.mTileM << "," << std::endl;
+  ss << "mTileN=" << options.mTileN << "," << std::endl;
   ss << "mTileScheduler=" << "gemm::TileScheduler(" << static_cast<int32_t>(options.mTileScheduler)
-     << ")" << std::endl;
+     << ")" << "," << std::endl;
+  ss << "mTransposeMmaOutput=" << options.mTransposeMmaOutput << "," << std::endl;
+  ss << "mUseCustomMmaSchedule=" << options.mUseCustomMmaSchedule << "," << std::endl;
+  ss << "mUseDeepSeekFp8=" << options.mUseDeepSeekFp8 << "," << std::endl;
+  ss << "mUseHoistTryWaitForCustomMmaSchedule=" << options.mUseHoistTryWaitForCustomMmaSchedule
+     << "," << std::endl;
+  ss << "mUsePerTokenSfA=" << options.mUsePerTokenSfA << "," << std::endl;
+  ss << "mUsePerTokenSfB=" << options.mUsePerTokenSfB << "," << std::endl;
+  ss << "mUseShuffledMatrixA=" << options.mUseShuffledMatrixA << "," << std::endl;
+  ss << "mUseTmaStore=" << options.mUseTmaStore << "," << std::endl;
+  ss << "mUseTwoTmaLoadWarps=" << options.mUseTwoTmaLoadWarps << "," << std::endl;
+  ss << "mUseTwoMmaWarps=" << options.mUseTwoMmaWarps << "," << std::endl;
+  ss << "mUseUnrollLoop2xForMma=" << options.mUseUnrollLoop2xForMma << "," << std::endl;
+  ss << "mWorldSize=" << options.mWorldSize << std::endl;
   return ss.str();
 }
 
@@ -493,8 +521,10 @@ inline int32_t getShuffleBlockSize(int epilogueTileM) {
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 // Check if the options are valid or not.
-inline bool checkAndUpdateGemmOptions(GemmOptions& options, bool isBlackwell, int /* tpGrpSize */,
+inline bool checkAndUpdateGemmOptions(GemmOptions& options, bool isBlackwell, int tpGrpSize,
                                       bool updateOptions = true) {
+  options.mWorldSize = tpGrpSize;
+
   if (options.mDtypeB == tg::Dtype::Void) {
     if (updateOptions) {
       options.mDtypeB = options.mDtypeA;
@@ -524,7 +554,8 @@ inline bool checkAndUpdateGemmOptions(GemmOptions& options, bool isBlackwell, in
   TLLM_CHECK_ERROR(
       (options.mDtypeA == options.mDtypeMmaA) ||
           ((options.mDtypeA == tg::Dtype::MxE2m1 || options.mDtypeA == tg::Dtype::E2m1) &&
-           options.mDtypeMmaA == tg::Dtype::Bfloat16),
+           options.mDtypeMmaA == tg::Dtype::Bfloat16) ||
+          (options.mDtypeA == tg::Dtype::E2m1 && options.mDtypeMmaA == tg::Dtype::E4m3),
       "Unsupported cast for A: ", tg::dtypeToString(options.mDtypeA), " -> ",
       tg::dtypeToString(options.mDtypeMmaA));
 
@@ -657,7 +688,16 @@ inline bool checkAndUpdateGemmOptions(GemmOptions& options, bool isBlackwell, in
   if (options.mMmaKind == tg::MmaKind::MxFp4NvFp4 || options.mMmaKind == tg::MmaKind::MxFp8Fp6Fp4) {
     TLLM_CHECK_ERROR(isBlackwell, "Block scaling is only supported on Blackwell");
 
-    int const mmaK = (options.mMmaKind == tg::MmaKind::MxFp4NvFp4) ? 64 : 32;
+    int mmaK = 32;
+    if (options.mMmaKind == tg::MmaKind::MxFp4NvFp4) {
+      if (options.mMmaK == 96) {
+        mmaK = 96;
+        TLLM_CHECK_ERROR(options.mTileK == 768, "When mmaK == 96, only tileK == 768 is supported");
+        TLLM_CHECK_ERROR(options.mTileN <= 128, "When mmaK == 96, only tileN <= 128 is supported");
+      } else {
+        mmaK = 64;
+      }
+    }
     if (options.mMmaK != mmaK) {
       int newTileK = mmaK * divUp(options.mTileK, mmaK);
       TLLM_LOG_WARNING("Unsupported MmaK (", options.mMmaK,
@@ -675,8 +715,26 @@ inline bool checkAndUpdateGemmOptions(GemmOptions& options, bool isBlackwell, in
     TLLM_CHECK_ERROR(options.mMmaN >= 64 || options.mMmaN == options.mTileN, "MmaN (",
                      options.mMmaN, ") must be >= 64 or equal to TileN (", options.mTileN, ")");
   }
+
+  if (options.mSfBlockSizeA.has_value()) {
+    // Only E2m1 x E4m3 is tested. MxE2m1 x bf16 may also work.
+    TLLM_CHECK_ERROR(options.mDtypeA == tg::Dtype::E2m1 && options.mDtypeB == tg::Dtype::E4m3,
+                     "sfBlockSizeA is only supported for E2m1 and E4m3 types. Found dtypeA=",
+                     tg::dtypeToString(options.mDtypeA),
+                     " dtypeB=", tg::dtypeToString(options.mDtypeB));
+
+    // sfBlockSizeA must be 16 or 32.
+    // SfBlockSizeA can also support 64 and 128, although they are not officially supported Nvida
+    // format. Note that the type conversion needs to happen before TCs.
+    // For example, convert e2m1 to e4m3 inside TmemCastA.
+    // If we want to support sfBlockSizeA=8, we can write another version of convertE2m1ToSfE4m3,
+    // which only packs 8 e2m1 elements.
+    TLLM_CHECK_ERROR(options.mSfBlockSizeA.value() == 16 || options.mSfBlockSizeA.value() == 32,
+                     "SfBlockSizeA (", options.mSfBlockSizeA.value(), ") must be 16 or 32.");
+  }
+
   if (tg::dtypeIsBlockFmt(options.mDtypeA)) {
-    int numEltsPerSfA = tg::dtypeNumEltsPerSf(options.mDtypeA);
+    int numEltsPerSfA = options.mSfBlockSizeA.value_or(tg::dtypeNumEltsPerSf(options.mDtypeA));
     TLLM_CHECK_ERROR(options.mTileK % (4 * numEltsPerSfA) == 0, "TileK (", options.mTileK,
                      ") must be a multiple of ", (4 * numEltsPerSfA), " for typeA ",
                      gemm::toString(options.mDtypeA));
@@ -823,6 +881,12 @@ inline bool checkAndUpdateGemmOptions(GemmOptions& options, bool isBlackwell, in
                    "Cluster DimZ is only allowed for split-k.");
   TLLM_CHECK_ERROR(options.mTileM <= 128, "GEMM does not support TileM > 128.");
 
+  // FIXME: this is a bug in DeepSeek Fp8.
+  if (options.mUseDeepSeekFp8) {
+    TLLM_CHECK_ERROR(options.mK % (options.mNumSlicesForSplitK * options.mTileK) == 0,
+                     "K must be a multiple of TileK * numSlicesForSplitK for DeepSeekFp8");
+  }
+
   // When the A-matrix is shuffled, the output must be transposed.
   if (options.mUseShuffledMatrixA) {
     // TODO add matrix shuffle for N-major epilogue.
@@ -1173,10 +1237,10 @@ inline bool checkAndUpdateGemmOptions(GemmOptions& options, bool isBlackwell, in
     // Init kernel traits.
     options.mKernelTraits = KernelTraits(
         options.mDtypeA, options.mDtypeB, options.mDtypeC, options.mDtypeAcc, options.mDtypeMmaA,
-        options.mDtypeMmaB, options.mMmaKind, options.mTileM, options.mTileN, options.mTileK,
-        options.mEpilogueTileM, options.mEpilogueTileN, options.mNumStages, options.mNumStagesMma,
-        options.mNumSlicesForSplitK, options.mNumSlicesForSliceK, options.mSplitK,
-        options.mUseTmaStore, options.mTransposeMmaOutput, options.mAllReduceAlgo,
+        options.mDtypeMmaB, options.mMmaKind, options.mMmaK, options.mTileM, options.mTileN,
+        options.mTileK, options.mEpilogueTileM, options.mEpilogueTileN, options.mNumStages,
+        options.mNumStagesMma, options.mNumSlicesForSplitK, options.mNumSlicesForSliceK,
+        options.mSplitK, options.mUseTmaStore, options.mTransposeMmaOutput, options.mAllReduceAlgo,
         options.mTileScheduler == TileScheduler::Persistent, options.mUseDeepSeekFp8,
         options.mUsePerTokenSfA, options.mUsePerTokenSfB, options.mBiasType);
   }
