@@ -17,10 +17,12 @@
 #pragma once
 
 #include <cassert>
+#include <stdexcept>
 
 #include "Enums.h"
 #include "trtllm/gen/CommonUtils.h"
 #include "trtllm/gen/DtypeDecl.h"
+#include "trtllm/gen/MmaDecl.h"
 
 namespace gemm {
 
@@ -66,16 +68,23 @@ class MemAllocatorHelper {
   }
 
   // Returns the offset of the ith chunk
-  int32_t getChunkOffset(int32_t ii) const {
-    if (mFirstChunkReuse[ii]) {
-      // Reuse the offset of the 0th chunk.
-      return getChunkOffset(0);
+  int32_t getChunkOffsetByName(std::string const& name) const {
+    for (size_t ii = 0; ii < mSmemChunkNames.size(); ++ii) {
+      if (mSmemChunkNames[ii] == name) {
+        return getChunkOffset(ii);
+      }
     }
+    throw std::runtime_error("Name not found: " + name);
+  }
 
-    // Get offset of ii chunks.
-    auto offset = getOffsetBeforeChunk(ii);
-    // Ensure alignment for the current chunk
-    return getSizePaddedToAlignment(offset, mNumBytesAndAlignmentPerSmemChunk[ii].second);
+  // Returns the first chunk reuse flag given chunk name.
+  int getFirstChunkReuseFlagByName(std::string const& name) const {
+    for (size_t ii = 0; ii < mSmemChunkNames.size(); ++ii) {
+      if (mSmemChunkNames[ii] == name) {
+        return getFirstChunkReuseFlag(ii);
+      }
+    }
+    throw std::runtime_error("Name not found: " + name);
   }
 
   // Function to calculate the total size of the SMEM array
@@ -83,9 +92,6 @@ class MemAllocatorHelper {
     return getOffsetBeforeChunk(static_cast<int32_t>(mNumBytesAndAlignmentPerSmemChunk.size()));
   }
 
-  // Returns the first chunk reuse flag for the ith chunk.
-  int getFirstChunkReuseFlag(int32_t ii) const { return mFirstChunkReuse[ii]; }
-
   // Print the contents of this object.
   void print() const {
     for (size_t ii = 0; ii < mNumBytesAndAlignmentPerSmemChunk.size(); ++ii) {
@@ -97,6 +103,21 @@ class MemAllocatorHelper {
   }
 
  private:
+  int32_t getChunkOffset(int32_t ii) const {
+    if (mFirstChunkReuse[ii]) {
+      // Reuse the offset of the 0th chunk.
+      return getChunkOffset(0);
+    }
+
+    // Get offset of ii chunks.
+    auto offset = getOffsetBeforeChunk(ii);
+    // Ensure alignment for the current chunk
+    return getSizePaddedToAlignment(offset, mNumBytesAndAlignmentPerSmemChunk[ii].second);
+  }
+
+  // Returns the first chunk reuse flag for the ith chunk.
+  int getFirstChunkReuseFlag(int32_t ii) const { return mFirstChunkReuse[ii]; }
+
   // Helper function to calculate padded size
   int32_t getSizePaddedToAlignment(int32_t size, int32_t alignment) const {
     assert((alignment & (alignment - 1)) == 0);
@@ -118,9 +139,7 @@ class MemAllocatorHelper {
 
 int getNumSmemBitsPerElt(tg::Dtype dtype, tg::MmaKind mmaKind) {
   if (mmaKind == tg::MmaKind::Auto) {
-    std::cout << "mmaKind != tg::MmaKind::Auto" << std::endl;
-    assert(false);
-    return -1;
+    throw std::runtime_error("mmaKind != tg::MmaKind::Auto");
   }
   if (mmaKind == tg::MmaKind::MxFp8Fp6Fp4) {
     return 8;
@@ -138,12 +157,13 @@ class KernelTraits {
 
   // The constructor.
   KernelTraits(tg::Dtype dtypeA, tg::Dtype dtypeB, tg::Dtype dtypeC, tg::Dtype dtypeAcc,
-               tg::Dtype dtypeMmaA, tg::Dtype dtypeMmaB, tg::MmaKind mmaKind, int32_t tileM,
-               int32_t tileN, int32_t tileK, int32_t epilogueTileM, int32_t epilogueTileN,
-               int32_t numStages, int32_t numStagesMma, int32_t numSlicesForSplitK,
-               int32_t numSlicesForSliceK, SplitK splitK, bool useTmaStore, bool transposeMmaOutput,
-               AllReduceAlgo allReduceAlgo, bool usePersistentScheduler, bool useDeepSeekFp8,
-               bool usePerTokenSfA, bool usePerTokenSfB, BiasType biasType)
+               tg::Dtype dtypeMmaA, tg::Dtype dtypeMmaB, tg::MmaKind mmaKind, int32_t mmaK,
+               int32_t tileM, int32_t tileN, int32_t tileK, int32_t epilogueTileM,
+               int32_t epilogueTileN, int32_t numStages, int32_t numStagesMma,
+               int32_t numSlicesForSplitK, int32_t numSlicesForSliceK, SplitK splitK,
+               bool useTmaStore, bool transposeMmaOutput, AllReduceAlgo allReduceAlgo,
+               bool usePersistentScheduler, bool useDeepSeekFp8, bool usePerTokenSfA,
+               bool usePerTokenSfB, BiasType biasType)
       : mMmaKind{mmaKind} {
     //
     // SMEM
@@ -445,8 +465,10 @@ class KernelTraits {
         // Number of columns for scaling factors of A.
         auto const numTmemColsSfA =
             useConstSfA
-                ? tg::roundUp((tileK / 64) * 2 * tg::ceilDiv(tileM, 64), 4)
-                : (useBlockScalingA ? ((tileK / 64) * 2 * tg::ceilDiv(tileM, 64)) * numStages : 0);
+                ? tg::roundUp((tileK / 64) * tg::getTmemColStridePerGroup(tileM, mmaK), 4)
+                : (useBlockScalingA
+                       ? ((tileK / 64) * tg::getTmemColStridePerGroup(tileM, mmaK)) * numStages
+                       : 0);
         // Number of columns for Sf alignment.
         auto const numColsAlignmentSfA = 4;
         // No need to reuse TMEM.
@@ -468,8 +490,10 @@ class KernelTraits {
         // Number of columns for scaling factors of B.
         auto const numTmemColsSfB =
             useConstSfB
-                ? tg::roundUp((tileK / 64) * 2 * tg::ceilDiv(tileN, 64), 4)
-                : (useBlockScalingB ? ((tileK / 64) * 2 * tg::ceilDiv(tileN, 64)) * numStages : 0);
+                ? tg::roundUp((tileK / 64) * tg::getTmemColStridePerGroup(tileN, mmaK), 4)
+                : (useBlockScalingB
+                       ? ((tileK / 64) * tg::getTmemColStridePerGroup(tileN, mmaK)) * numStages
+                       : 0);
         // Number of columns for Sf alignment.
         auto const numColsAlignmentSfB = 4;
         // No need to reuse TMEM.
@@ -516,13 +540,13 @@ inline int32_t getTmemBufferSize(KernelTraits traits) {
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetLoadA(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(0);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemLoadA");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetLoadB(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(1);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemLoadB");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -532,56 +556,56 @@ inline int32_t getSmemOffsetLoadAb(KernelTraits traits) { return getSmemOffsetLo
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetLoadShuffleB(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(2);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemBShuffle");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetGmemC(KernelTraits traits, int resIdx = 0) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(3 + resIdx);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemGmemC" + std::to_string(resIdx));
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetRowMax(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(5);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemRowMax");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetSliceK(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(6);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemSliceK");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetPerTokenSf(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(7);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemPerTokenSf");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetBias(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(8);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemBias");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetBlockAmax(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(9);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemBlockAmax");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getSmemOffsetConstSfBuf(KernelTraits traits) {
-  return traits.mSmemAllocatorHelper.getChunkOffset(10);
+  return traits.mSmemAllocatorHelper.getChunkOffsetByName("smemConstSfBuf");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t isSmemAbRepurposedToGmemC(KernelTraits traits, int resIdx = 0) {
-  // Be conscious that the index (3 + resIdx) should match the index in getSmemOffsetGmemC().
-  return traits.mSmemAllocatorHelper.getFirstChunkReuseFlag(3 + resIdx);
+  return traits.mSmemAllocatorHelper.getFirstChunkReuseFlagByName("smemGmemC" +
+                                                                  std::to_string(resIdx));
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -591,25 +615,25 @@ inline int32_t isSmemAbRepurposedToGmemC(KernelTraits traits, int resIdx = 0) {
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getTmemOffsetD(KernelTraits traits) {
-  return traits.mTmemAllocatorHelper.getChunkOffset(0);
+  return traits.mTmemAllocatorHelper.getChunkOffsetByName("tmemD");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getTmemOffsetA(KernelTraits traits) {
-  return traits.mTmemAllocatorHelper.getChunkOffset(1);
+  return traits.mTmemAllocatorHelper.getChunkOffsetByName("tmemA");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getTmemOffsetSfA(KernelTraits traits) {
-  return traits.mTmemAllocatorHelper.getChunkOffset(2);
+  return traits.mTmemAllocatorHelper.getChunkOffsetByName("tmemSfA");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 inline int32_t getTmemOffsetSfB(KernelTraits traits) {
-  return traits.mTmemAllocatorHelper.getChunkOffset(3);
+  return traits.mTmemAllocatorHelper.getChunkOffsetByName("tmemSfB");
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
