@@ -16,33 +16,26 @@ limitations under the License.
 
 import pytest
 import torch
-from jit_utils import jit_prefill_attention_func_args
+from jit_utils import gen_prefill_attention_modules
 
 import flashinfer
 
 
 @pytest.fixture(autouse=True, scope="module")
 def warmup_jit():
-    if flashinfer.jit.has_prebuilt_ops:
-        yield
-    else:
-        try:
-            flashinfer.jit.parallel_load_modules(
-                jit_prefill_attention_func_args(
-                    [torch.float16],  # q_dtypes
-                    [torch.float16],  # kv_dtypes
-                    [64, 128, 256],  # head_dims
-                    [0],  # pos_encoding_modes
-                    [False],  # use_sliding_windows
-                    [False],  # use_logits_soft_caps
-                    [False],  # use_fp16_qk_reductions
-                )
-            )
-        except Exception as e:
-            # abort the test session if warmup fails
-            pytest.exit(str(e))
-        finally:
-            yield
+    flashinfer.jit.build_jit_specs(
+        gen_prefill_attention_modules(
+            [torch.float16],  # q_dtypes
+            [torch.float16],  # kv_dtypes
+            [64, 128, 256],  # head_dims
+            [0],  # pos_encoding_modes
+            [False],  # use_sliding_windows
+            [False],  # use_logits_soft_caps
+            [False],  # use_fp16_qk_reductions
+        ),
+        verbose=False,
+    )
+    yield
 
 
 @pytest.mark.parametrize("seq_len", [1, 7, 127, 999, 3579])
@@ -197,7 +190,7 @@ def test_batch_paged_prefill_packed_input(
     q = q.view(-1, num_qo_heads, head_dim)
     o_packed = wrapper.run(q, paged_kv_cache)
     o_contiguous = wrapper.run(q.contiguous(), paged_kv_cache)
-    torch.testing.assert_close(o_packed, o_contiguous, rtol=1e-3, atol=1e-3)
+    torch.testing.assert_close(o_packed, o_contiguous, rtol=1e-3, atol=2e-3)
 
 
 if __name__ == "__main__":
